"use strict";

{
	const DOM_COMPONENT_ID = "mobileiap";

	function Log(str)
	{
		console.log("[C3 IAP]", str);
	}

	const HANDLER_CLASS = class IAPDOMHandler extends self.DOMHandler
	{
		constructor(iRuntime)
		{
			super(iRuntime, DOM_COMPONENT_ID);
			
			this._store = null;
			this._products = [];
			this._pendingProducts = new Set;
			this._lastError = null;

			this.AddRuntimeMessageHandler("init", e => this._Initialise(e));
			this.AddRuntimeMessageHandler("register", o => this._OnRegister(o));
			this.AddRuntimeMessageHandler("complete_registration", _ => this._OnCompleteRegistration());
			this.AddRuntimeMessageHandler("purchase", id => this._OnPurchase(id));
			this.AddRuntimeMessageHandler("restore", _ => this._OnRestore());
		}

		// helpers

		_Initialise(e)
		{
			Log("Initialising");
			
			this._store = window["store"];
			
			if (!this._store)
				return;
			
			const validatorUrl = e["validator-url"];
			if (validatorUrl)
				this._store["validator"] = validatorUrl;
			
			this._store["ready"](_ => this._OnReady());
			this._store["error"](err => this._OnGenericError(err));
			this._When("valid", product => this._OnProductAvailable(product));
			this._When("approved", product => this._OnProductApproved(product));
			this._When("verified", product => this._OnProductVerified(product));
			this._When("finished", product => this._OnPurchaseSuccess(product));
			this._When("owned", product => this._OnProductOwned(product));
			// NOTE only triggers when a product is invalid
			this._When("error", (err, product) => this._OnPurchaseFail(product, err));
		}
		
		_When(verb, callback)
		{
			this._store["when"]("product")[verb](callback);
		}

		// events

		_OnReady ()
		{
			Log("store ready");
			// products is a modified array with appended functions, so it
			// doesn't travel well. copy it to a new array to fix this
			const products = [...this._store["products"]];

			this.PostToRuntime("registration", products);
		}

		_OnProductApproved (product)
		{
			Log(`product "${product["alias"]}" approved, verifying`);
			product["verify"]();		// approved -> verify
		}

		_OnProductVerified (product)
		{
			Log(`product "${product["alias"]}" verified, finishing`);
			product["finish"]();		// verified -> finish
		}

		_OnProductAvailable (product)
		{
			if (this._pendingProducts.has(product["id"]))
				this._OnPurchaseFail(product, this._lastError);
			this.PostToRuntime("product-available", product);
		}

		_OnProductOwned (product)
		{
			this.PostToRuntime("product-owned", product);
		}

		_OnPurchaseSuccess (product)
		{
			this._pendingProducts.delete(product["id"]);
			this.PostToRuntime("purchase-success", product);
		}

		_OnPurchaseFail (product, err)
		{
			this.PostToRuntime("purchase-failure", [product, err]);
		}

		_OnGenericError (err)
		{
			this._lastError = err;
			Log(err);
		}

		// actions

		_OnRegister(o)
		{
			const id = o["id"];
			const type = o["type"];

			if (!this._store)
				return null;
				
			this._products.push({
				"id": id,
				"alias": id,
				"type": this._store[type]
			});
		}

		_OnCompleteRegistration()
		{
			if (!this._store)
				return;
			
			for (const product of this._products)
			{
				Log(`Registering product "${product["alias"]}"`);
				this._store["register"](product);
			}
			
			this._store["refresh"]();
		}

		_OnPurchase(id)
		{
			if (!this._store)
				return;

			const product = this._store["get"](id);

			// if the product is invalid it will helpfully cause the application to crash
			if (!product["valid"])
			{
				const err = {
					"code": -1,
					"message": "Unable to purchase invalid product"
				};
				this.PostToRuntime("purchase-failure", [product, err]);
				return;
			}

			this._pendingProducts.add(id);
			Log(`Purchasing product "${id}"`);
			this._store["order"](id);
		}

		_OnRestore()
		{
			if (!this._store)
				return;
				
			this._store["refresh"]();
		}
	};

	self.RuntimeInterface.AddDOMHandlerClass(HANDLER_CLASS);
}