"use strict";

{
	const DOM_COMPONENT_ID = "mobileiap";

	function Log(str)
	{
		console.log("[C3 IAP]", str);
	}

	const HANDLER_CLASS = class IAPDOMHandler extends self.DOMHandler
	{
        _store: CdvPurchase.Store;
        _products: CdvPurchase.IRegisterProduct[];

		constructor(iRuntime)
		{
			super(iRuntime, DOM_COMPONENT_ID);

			this._store = null;
			this._products = [];
			this._pendingProducts = new Set;
			this._lastError = null;

			this.AddRuntimeMessageHandler("init", e => this._Initialise(e));
			this.AddRuntimeMessageHandler("register", o => this._OnRegister(o));
			this.AddRuntimeMessageHandler("complete_registration", _ => this._OnCompleteRegistration());
			this.AddRuntimeMessageHandler("purchase", id => this._OnPurchase(id));
			this.AddRuntimeMessageHandler("restore", _ => this._OnRestore());
		}

		// helpers

		_Initialise(e)
		{
			Log("Initialising");

			if (window["CdvPurchase"])
				this._store = window["CdvPurchase"]["store"];

			if (!this._store)
				return;

			// Uncomment to list all debug logs
			//this._store["verbosity"] = 4;

			const validatorUrl = e["validator-url"];
			if (validatorUrl)
				this._store["validator"] = validatorUrl;

			this._store["ready"](_ => this._OnReady());
			this._store["error"](err => this._OnGenericError(err));

			this._store["when"]()
				["productUpdated"](product => this._OnProductUpdated(product))
				["approved"](transaction => this._OnTransactionApproved(transaction))
				["verified"](verifiedReceipt => this._OnTransactionVerified(verifiedReceipt))
				["unverified"](param => this._OnUnverified(param))
				["finished"](transaction => this._OnTransactionFinished(transaction))
		}

		// Convert a Product class from the cordova-plugin-purchase API to the minimal JSON
		// data necessary to post to the runtime.
		_ToProductInfo(product: CdvPurchase.Product)
		{
			return {
				"id": product["id"],
				"owned": product["owned"],
				"canPurchase": product["canPurchase"],

				"title": product["title"],
				"description": product["description"],
				"price": product["pricing"]["price"],

				"transaction": null	// TODO
			};
		}

		// events

		_OnReady ()
		{
			Log("store ready");

			this.PostToRuntime("registration", this._store["products"].map(p => this._ToProductInfo(p)));
		}

		_OnTransactionApproved(transaction: CdvPurchase.Transaction)
		{
			Log(`product "${transaction["products"].map(p => p["id"]).join(",")}" approved, verifying`);

			// TODO: doesn't work??
			//transaction["verify"]();		// approved -> verify

			transaction["finish"]();		// approved -> finish
		}

		// TODO: currently dead code
		_OnTransactionVerified(verifiedReceipt)
		{
			Log(`transaction verified, finishing`);
			verifiedReceipt["finish"]();		// verified -> finish

			/*
			for (const verifiedPurchase of verifiedReceipt["collection"])
			{
				const productId = verifiedPurchase["id"];

				if (this._store["owned"](productId))
				{
					Log(`product ${productId} now owned`);
					const product = this._store["get"](productId);
					this._OnProductOwned(product);
				}
			};
			*/
		}

		// TODO: currently dead code
		_OnUnverified(param)
		{
			console.log("IAP unverified event: ", param);
		}

		_OnProductUpdated(product)
		{
			if (product["owned"])
				this._OnProductOwned(product);
			else
				this._OnProductAvailable(product);
		}

		_OnProductAvailable(product)
		{
			this.PostToRuntime("product-available", this._ToProductInfo(product));
		}

		_OnProductOwned(product)
		{
			this.PostToRuntime("product-owned", this._ToProductInfo(product));
		}

		_OnTransactionFinished(transaction)
		{
			Log(`transaction finished`);

			for (const prodInfo of transaction["products"])
			{
				this._pendingProducts.delete(prodInfo["id"]);
				const product = this._store["get"](prodInfo["id"]);
				this._OnProductOwned(product);
			}
		}

		_OnPurchaseSuccess(product)
		{
			this._pendingProducts.delete(product["id"]);
			this.PostToRuntime("purchase-success", this._ToProductInfo(product));
		}

		_OnPurchaseFail(product, err)
		{
			this.PostToRuntime("purchase-failure", [this._ToProductInfo(product), err]);
		}

		_OnGenericError(err)
		{
			this._lastError = err;
			console.error("[C3 IAP] Store error: ", err);
		}

		// actions

		_OnRegister(o)
		{
			const id = o["id"];
			const type = o["type"];

			if (!this._store)
				return null;

			this._products.push({
				"id": id,
				// "alias": id,
				"type": type,
				"platform": this._store["defaultPlatform"]()
			});
		}

		_OnCompleteRegistration()
		{
			if (!this._store)
				return;

			Log(`Registering ${this._products.length} products`);

			this._store["register"](this._products);

			this._store["initialize"]();
		}

		_OnPurchase(id)
		{
			if (!this._store)
				return;

			const product = this._store["get"](id);

			// Invalid products definitely can't be purchased, so error out early
			if (!product)
			{
				const err = {
					"code": -1,
					"message": "Unable to purchase invalid product"
				};
				this.PostToRuntime("purchase-failure", [this._ToProductInfo(product), err]);
				return;
			}

			this._pendingProducts.add(id);
			Log(`Purchasing product "${id}"`);
			this._store["order"](product["getOffer"]())
			.then(result =>
			{
				if (result && result["isError"])
				{
					this._OnPurchaseFail(product, result);
				}
				else
				{
					this._OnPurchaseSuccess(product);
				}
			})
		}

		_OnRestore()
		{
			if (!this._store)
				return;

			this._store["restorePurchases"]();
		}
	};

	self.RuntimeInterface.AddDOMHandlerClass(HANDLER_CLASS);
}
